//////////////////////////////////////////////////////
// Subversion - Gemini connector application
// by: Frans Bouma
// http://www.llblgen.com
// http://weblogs.asp.net/fbouma
// 
// Special thanks to: Kognition (http://www.syringe.net.nz/PermaLink,guid,b0fa47ff-8853-4c94-b823-431f73e30ec1.aspx)
// for giving me the idea how to get started. 
//
// License: BSD2 license. 
//////////////////////////////////////////////////////
// COPYRIGHTS:
// Copyright (c)2006 Solutions Design. All rights reserved.
// 
// This source-code is released under the following license: (BSD2)
// -------------------------------------------
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met: 
//
// 1) Redistributions of source code must retain the above copyright notice, this list of 
//    conditions and the following disclaimer. 
// 2) Redistributions in binary form must reproduce the above copyright notice, this list of 
//    conditions and the following disclaimer in the documentation and/or other materials 
//    provided with the distribution. 
// 
// THIS SOFTWARE IS PROVIDED BY SOLUTIONS DESIGN ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOLUTIONS DESIGN OR CONTRIBUTORS BE LIABLE FOR 
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
// The views and conclusions contained in the software and documentation are those of the authors 
// and should not be interpreted as representing official policies, either expressed or implied, 
// of Solutions Design. 
//
//////////////////////////////////////////////////////
// Requirements: .NET 2.0, Subversion 1.x, Gemini v3.x, WSE 3.x runtime installed on server
//////////////////////////////////////////////////////
using System;
using System.Text;
using System.IO;
using CounterSoft.Gemini.Commons;
using CounterSoft.Gemini.Commons.Entity;
using CounterSoft.Gemini.Logging;

namespace CounterSoft.Gemini.SourceControl.SVN
{
	/// <summary>
	/// Handler class for post-commit actions. Used when called from a subversion post-commit hook.
	/// </summary>
	internal class PostCommitHandler
	{
		/// <summary>
		/// Handles the post-commit action on the repository path specified with the revision specified. Called after the commit transaction has been
		/// completed.
		/// </summary>
		/// <param name="repositoryPath">The repository path the commit is issued on.</param>
		/// <param name="revisionNumber">The revision number.</param>
		/// <returns>0 if succeeded, 1 otherwise</returns>
		/// <remarks>This routine logs the comments and other info of the revision commit to the issues specified inside the comments. If this
		/// fails the routine will write the errors to stderr, however subversion will ignore these errors.</remarks>
		internal int Handle( string repositoryPath, int revisionNumber )
		{
            try
            {
                // get the checkin comment, and parse it for gemini issueIDs. If no IDs are specified, exit.
                string checkInComment = SvnLookCaller.GetCheckinComment(repositoryPath, revisionNumber);

                int[] issueIDs = Utilities.ParseCheckinCommentForIssueIDs(checkInComment);

                if (issueIDs.Length <= 0)
                {
                    // none found, abort.
                    DiagnosticsManager.TraceMessage("No Gemini Issue ID's were specified in the check-in comment, so no Gemini issue could be linked to this revision. Aborting.");
                    DiagnosticsManager.TraceMessage("Comment: " + checkInComment);
#if DEBUG
                    DiagnosticsManager.EventLogMessage("No Gemini Issue ID's were specified in the check-in comment, so no Gemini issue could be linked to this revision. Aborting.");
                    DiagnosticsManager.EventLogMessage("Comment: " + checkInComment);
#endif
                    return 0;
                }

                foreach (int issueID in issueIDs)
                {
                    DiagnosticsManager.TraceMessage("Issue ID: " + issueID);
                }

                DiagnosticsManager.TraceMessage("Commit Comment: " + checkInComment);
                DiagnosticsManager.TraceMessage("Gemini User ID: " + Program.GeminiServiceManager.UsersService.WhoAmI().UserName);
#if DEBUG
                DiagnosticsManager.EventLogMessage("Commit Comment: " + checkInComment);
                DiagnosticsManager.EventLogMessage("Gemini User ID: " + Program.GeminiServiceManager.UsersService.WhoAmI().UserName);
#endif
                string issueTemplate = string.Empty;
                string templatePathFilename = Path.Combine(Path.Combine(repositoryPath, "hooks"), "revisionLogTemplate.txt");

                if (!File.Exists(templatePathFilename))
                {
                    DiagnosticsManager.TraceMessage("Template file 'revisionLogTemplate.txt' not found. Aborting");
                    return 0;
                }

                StreamReader reader = new StreamReader(templatePathFilename);
                issueTemplate = reader.ReadToEnd();
                reader.Close();

                string author = SvnLookCaller.GetAuthor(repositoryPath, revisionNumber);
                string committedFiles = SvnLookCaller.GetCommittedFiles(repositoryPath, revisionNumber);

                repositoryPath = repositoryPath.Trim();

                DiagnosticsManager.TraceMessage("SVN check-in Author: " + author);
                DiagnosticsManager.TraceMessage("SVN check-in Repository Path: " + repositoryPath);
                DiagnosticsManager.TraceMessage("SVN check-in Revision Number: " + revisionNumber);
                DiagnosticsManager.TraceMessage("SVN check-in Committed Files: " + committedFiles);

                // build the gemini issue comment to add to all issues specified
                StringBuilder commentToAdd = new StringBuilder(issueTemplate);
                commentToAdd = commentToAdd.Replace("$AUTHOR", author);

                //commentToAdd = commentToAdd.Replace( "$FILES", committedFiles.Replace( "\r\n", "<br/>" ) );
                commentToAdd = commentToAdd.Replace("$FILES",
                    Utilities.GetFormattedHtmlFileList(SvnLookCaller.GetCommittedFilesCollection(repositoryPath, revisionNumber)));

                commentToAdd = commentToAdd.Replace("$CHECKINCOMMENT", checkInComment.Replace("\r\n", "<br/>"));
                commentToAdd = commentToAdd.Replace("$REVISION", revisionNumber.ToString());

                IssueCommentEN sourceControlComment = new IssueCommentEN();
                SourceControlEN sourceControlFileLink = new SourceControlEN();

                sourceControlComment.ProjectID = 0;
                sourceControlComment.UserID = Program.GeminiServiceManager.UsersService.WhoAmI().UserID;
                sourceControlComment.Comment = commentToAdd.ToString();
                sourceControlComment.DateCreated = DateTime.Now;

                // Process each issue id specified
                foreach (int issueID in issueIDs)
                {
                    if (Properties.Settings.Default.AddIssueComment)
                    {
                        sourceControlComment.IssueID = issueID;
                        sourceControlComment.ProjectID = Program.GeminiServiceManager.IssuesService.GetIssue(issueID).ProjectID; 

                        Program.GeminiServiceManager.IssuesService.CreateComment(issueID, sourceControlComment);
                    }

                    if (Properties.Settings.Default.AddIssueSourceCodeFileLink)
                    {
                        committedFiles = committedFiles.Replace("\r\n", "|");
                        string[] tempFiles = committedFiles.Split('|');

                        foreach (string file in tempFiles)
                        {
                            string f = file.Trim();

                            if (repositoryPath.Length > 0 && f.Length > 0)
                            {
                                string changeOperation = "";
                                string changeMessage = "";
                                string filename = f;
                                string filepath = f;

                                if (f.Length > 3)
                                {
                                    changeOperation = f.Substring(0, 2).ToUpperInvariant();

                                    f = f.Substring(2).Trim();

                                    filename = Path.GetFileName(f);
                                    filepath = Path.GetDirectoryName(f);
                                }

                                switch (changeOperation.ToUpperInvariant())
                                {
                                    case "A ":
                                        changeMessage = "Added";
                                        break;
                                    case "D ":
                                        changeMessage = "Deleted";
                                        break;
                                    case "U ":
                                        changeMessage = "Updated";
                                        break;
                                    case " U":
                                        changeMessage = "Properties";
                                        break;
                                    case "UU":
                                        changeMessage = "Updated + Properties";
                                        break;
                                }

                                if (changeMessage.Length > 0)
                                    changeMessage = string.Format(" ({0})", changeMessage);

                                sourceControlFileLink = new SourceControlEN();
                                sourceControlFileLink.DateCreated = DateTime.Now;
                                sourceControlFileLink.SourceControlRepository = repositoryPath;
                                sourceControlFileLink.IssueID = issueID;
                                //sourceControlFileLink.FileName = string.Format("{0} {1}", filename, changeMessage);
                                sourceControlFileLink.FileName = filename;
                                sourceControlFileLink.FilePath = filepath;

                                Program.GeminiServiceManager.IssuesService.CreateSourceControlFile(issueID, sourceControlFileLink);
                            }
                            else
                            {
                                DiagnosticsManager.TraceMessage("Empty committed file data?");
                            }
                        }
                    }
                }

                // all Ok
                return 0;

            }
            catch (Exception ex)
            {
                DiagnosticsManager.TraceException("Gemini PostCommitHandler.Handle", ex);
#if DEBUG
                DiagnosticsManager.EventLogException("Gemini PostCommitHandler.Handle", ex);
#endif
                return 0;
            }
        }
	}
}
