//////////////////////////////////////////////////////
// Subversion - Gemini connector application
// by: Frans Bouma
// http://www.llblgen.com
// http://weblogs.asp.net/fbouma
// 
// Special thanks to: Kognition (http://www.syringe.net.nz/PermaLink,guid,b0fa47ff-8853-4c94-b823-431f73e30ec1.aspx)
// for giving me the idea how to get started. 
//
// License: BSD2 license. 
//////////////////////////////////////////////////////
// COPYRIGHTS:
// Copyright (c)2006 Solutions Design. All rights reserved.
// 
// This source-code is released under the following license: (BSD2)
// -------------------------------------------
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met: 
//
// 1) Redistributions of source code must retain the above copyright notice, this list of 
//    conditions and the following disclaimer. 
// 2) Redistributions in binary form must reproduce the above copyright notice, this list of 
//    conditions and the following disclaimer in the documentation and/or other materials 
//    provided with the distribution. 
// 
// THIS SOFTWARE IS PROVIDED BY SOLUTIONS DESIGN ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOLUTIONS DESIGN OR CONTRIBUTORS BE LIABLE FOR 
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
// The views and conclusions contained in the software and documentation are those of the authors 
// and should not be interpreted as representing official policies, either expressed or implied, 
// of Solutions Design. 
//
//////////////////////////////////////////////////////
// Requirements: .NET 2.0, Subversion 1.x, Gemini v2.x, WSE 2.x runtime installed on server
//////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Text;
using CounterSoft.Gemini.Commons;
using CounterSoft.Gemini.Logging;
using CounterSoft.Gemini.Commons.Entity;

namespace CounterSoft.Gemini.SourceControl.SVN
{
	/// <summary>
	/// Handler class for pre-commit actions. Used when called from a subversion pre-commit hook.
	/// </summary>
	internal class PreCommitHandler
	{
		/// <summary>
		/// Handles the pre-commit action on the repository path specified with the transaction with the given name.
		/// </summary>
		/// <param name="repositoryPath">The repository path the commit is issued on.</param>
		/// <param name="transactionName">Name of the transaction which this commit started.</param>
		/// <returns>0 if the commit can proceed, 1 otherwise. If 1 is returned, the error is logged to stderr so Subversion will re-route it
		/// back to the client app.</returns>
		/// <remarks>This routine simply checks if the commit can proceed. It logs an error if that's not the case and returns 1 if the commit has to be
		/// aborted, otherwise 0 is returned to signal subversion that everything is allright and the commit can proceed. Uses Svnlook to examine
		/// the transaction details.</remarks>
		internal int Handle( string repositoryPath, string transactionName )
		{
            DiagnosticsManager.TraceMessage("PreCommitHandler.Handle Started.");
            DiagnosticsManager.TraceMessage("PreCommitHandler.Handle Path: " + repositoryPath);
            DiagnosticsManager.TraceMessage("PreCommitHandler.Handle Transaction: " + transactionName);

            if(!Properties.Settings.Default.PerformPreCommitChecks)
			{
				// no checks have to be performed.
				return 0;
			}

			// get the checkin comment, and parse it for gemini issueIDs. If no IDs are specified or non-existing IDs, pre-commit will fail.
			string checkInComment = SvnLookCaller.GetCheckinComment(repositoryPath, transactionName );

			int[] issueIDs = Utilities.ParseCheckinCommentForIssueIDs( checkInComment );

			if( issueIDs.Length <= 0 )
			{
                DiagnosticsManager.TraceMessage("No Gemini Issue IDs were specified in the check-in comment, so no Gemini issue could be linked to this commit. Aborting.");
                DiagnosticsManager.TraceMessage("Comment: " + checkInComment);
                return 0;
			}

			// check if the issues exist.
			List<int> nonExistingIssueIds = new List<int>();

			foreach( int issueID in issueIDs )
			{
				IssueEN issue = GetIssue(issueID);
				if( issue == null )
				{
					nonExistingIssueIds.Add( issueID );
				}
			}

			if(nonExistingIssueIds.Count>0)
			{
				StringBuilder error = new StringBuilder("The following specified issueIDs couldn't be found in your Gemini system: ");
				for( int i = 0; i < nonExistingIssueIds.Count; i++ )
				{
					if( i > 0 )
					{
						error.Append( ", " );
					}
					error.Append( nonExistingIssueIds[i] );
				}
				error.Append( ". Aborting" );

                DiagnosticsManager.TraceMessage(error.ToString());
				return 0;
			}

			// all clear, let the commit proceed.

            DiagnosticsManager.TraceMessage("PreCommitHandler.Handle Finished.");
           
            return 0;
		}


		/// <summary>
		/// Gets the issue object of the issueid specified.
		/// </summary>
		/// <param name="issueID">The issue ID.</param>
		/// <returns>the issue object with the issue id specified, or null if not found</returns>
		private IssueEN GetIssue(int issueID)
		{
            DiagnosticsManager.TraceMessage("GetIssue...");

			IssueEN toReturn = null;

			try
			{
                toReturn = Program.GeminiServiceManager.IssuesService.GetIssue(issueID);
			}
			catch (Exception ex)
			{
                DiagnosticsManager.TraceException("GetIssue", ex);
			}

            DiagnosticsManager.TraceMessage("GetIssue done.");

            return toReturn;
		}
	}
}
