//////////////////////////////////////////////////////
// Subversion - Gemini connector application
// by: Frans Bouma
// http://www.llblgen.com
// http://weblogs.asp.net/fbouma
// 
// Special thanks to: Kognition (http://www.syringe.net.nz/PermaLink,guid,b0fa47ff-8853-4c94-b823-431f73e30ec1.aspx)
// for giving me the idea how to get started. 
//
// License: BSD2 license. 
//////////////////////////////////////////////////////
// COPYRIGHTS:
// Copyright (c)2006 Solutions Design. All rights reserved.
// 
// This source-code is released under the following license: (BSD2)
// -------------------------------------------
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met: 
//
// 1) Redistributions of source code must retain the above copyright notice, this list of 
//    conditions and the following disclaimer. 
// 2) Redistributions in binary form must reproduce the above copyright notice, this list of 
//    conditions and the following disclaimer in the documentation and/or other materials 
//    provided with the distribution. 
// 
// THIS SOFTWARE IS PROVIDED BY SOLUTIONS DESIGN ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOLUTIONS DESIGN OR CONTRIBUTORS BE LIABLE FOR 
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
// The views and conclusions contained in the software and documentation are those of the authors 
// and should not be interpreted as representing official policies, either expressed or implied, 
// of Solutions Design. 
//
//////////////////////////////////////////////////////
// Requirements: .NET 2.0, Subversion 1.x, Gemini v3.x installed on server
//////////////////////////////////////////////////////
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using CounterSoft.Gemini.Logging;

namespace CounterSoft.Gemini.SourceControl.SVN
{
	/// <summary>
	/// Class which calls svnlook with various parameters and returns the gathered values. Requires a filled ConfigurationData object.
	/// </summary>
	/// <remarks>Code based on Kognition's svnlook caller</remarks>
	internal static class SvnLookCaller
	{
		/// <summary>
		/// Gets the author who checked in the revision passed in
		/// </summary>
		/// <param name="repositoryPath">The repository.</param>
		/// <param name="revisionNumber">The revision.</param>
		/// <returns>name of the author who checked in the revision passed in</returns>
		internal static string GetAuthor(string repositoryPath, int revisionNumber)
		{
			return CallSvnLook(string.Format( "author \"{0}\" -r {1}", repositoryPath, revisionNumber ));
		}

		/// <summary>
		/// Gets the author who started the commit in progress 
		/// </summary>
		/// <param name="repositoryPath">The repository.</param>
		/// <param name="transactionName">Name of the transaction of the current commit</param>
		/// <returns>
		/// name of the author who started the commit in progress
		/// </returns>
		internal static string GetAuthor(string repositoryPath, string transactionName)
		{
            return CallSvnLook(string.Format("author \"{0}\" -t {1}", repositoryPath, transactionName));
		}

		/// <summary>
		/// Gets the checkin comment passed to the commit of revision passed in
		/// </summary>
		/// <param name="repositoryPath">The repository.</param>
		/// <param name="revisionNumber">The revision.</param>
		/// <returns>returns the commit passed to the commit of the revision passed in</returns>
		internal static string GetCheckinComment(string repositoryPath, int revisionNumber)
		{
            return CallSvnLook(string.Format("log \"{0}\" -r {1}", repositoryPath, revisionNumber));
		}


		/// <summary>
		/// Gets the checkin comment passed to the commit in progress
		/// </summary>
		/// <param name="repositoryPath">The repository.</param>
        /// <param name="transaction">Name of the transaction of the current commit</param>
		/// <returns>
		/// returns the checkin comment passed to the commit in progress
		/// </returns>
		internal static string GetCheckinComment(string repositoryPath, string transaction)
		{
            return CallSvnLook(string.Format("log \"{0}\" -t {1}", repositoryPath, transaction));
		}

		/// <summary>
		/// Gets the list of files of the commit of the revision passed in.
		/// </summary>
		/// <param name="repositoryPath">The repository.</param>
		/// <param name="revisionNumber">The revision.</param>
		/// <returns>returns the list of files of the commit of the revision passed in. This list contains on each line a change type specifier and the filename.</returns>
		internal static string GetCommittedFiles(string repositoryPath, int revisionNumber)
		{
            return CallSvnLook(string.Format("changed \"{0}\" -r {1}", repositoryPath, revisionNumber));
		}

        /// <summary>
        /// Gets the list of files of the commit of the revision passed in.
        /// </summary>
        /// <param name="repositoryPath">The repository.</param>
        /// <param name="revisionNumber">The revision.</param>
        /// <returns>returns the list of files of the commit of the revision passed in. This list contains on each line a change type specifier and the filename.</returns>
        internal static List<CommittedFileEN> GetCommittedFilesCollection(string repositoryPath, int revisionNumber)
        {
            List<CommittedFileEN> files = new List<CommittedFileEN>();

            string raw = CallSvnLook(string.Format("changed \"{0}\" -r {1}", repositoryPath, revisionNumber));

            raw = raw.Replace("\r\n", "|");
            
            string[] tempFiles = raw.Split('|');

            foreach (string file in tempFiles)
            {
                string f = file.Trim();

                if (repositoryPath.Length > 0 && f.Length > 0)
                {
                    string changeOperation = "";
                    string changeMessage = "";
                    string filename = f;
                    string filepath = f;

                    if (f.Length > 3)
                    {
                        changeOperation = f.Substring(0, 2).ToUpperInvariant();

                        f = f.Substring(2).Trim();

                        filename = Path.GetFileName(f);
                        filepath = Path.GetDirectoryName(f);
                    }

                    switch (changeOperation.ToUpperInvariant())
                    {
                        case "A ":
                            changeMessage = "Added";
                            break;
                        case "D ":
                            changeMessage = "Deleted";
                            break;
                        case "U ":
                            changeMessage = "Updated";
                            break;
                        case " U":
                            changeMessage = "Properties";
                            break;
                        case "UU":
                            changeMessage = "Updated + Properties";
                            break;
                    }

                    files.Add(new CommittedFileEN(filepath, filename, changeMessage, f));
                }
                else
                {
                    DiagnosticsManager.TraceMessage("Empty committed file data?");
                }
            }

            return files;
        }
		
		/// <summary>
		/// Calls Svnlook, passes the argument specified and returns the result reported by svnlook in string form.
		/// </summary>
		/// <param name="argument">The argument.</param>
		/// <returns>the output of svnlook</returns>
		private static string CallSvnLook(string argument)
		{
			Process svnLook = new Process();

            svnLook.StartInfo.StandardOutputEncoding = System.Text.Encoding.Default; 
			svnLook.StartInfo.UseShellExecute = false;
			svnLook.StartInfo.RedirectStandardOutput = true;
            svnLook.StartInfo.FileName = Path.Combine(Properties.Settings.Default.SVNBinPath, "svnlook.exe");
            svnLook.StartInfo.WorkingDirectory = Properties.Settings.Default.SVNBinPath;
			svnLook.StartInfo.Arguments = argument;
			
            svnLook.Start();
			
            string toReturn = svnLook.StandardOutput.ReadToEnd();

            if (toReturn != null)
            {
                toReturn = toReturn.TrimEnd();
            }
			
            svnLook.WaitForExit();
			
            return toReturn;
		}
	}
}
